#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

from pygameday import util
from pygameday.pygameday import GameDayClient

__all__ = ["GameDayClient"]

# Set up logging
util.configure_logging("pygameday")

