from distutils.core import setup
setup(
  name = 'pygameday',
  packages = ['pygameday'],
  version = '0.2',
  description = 'Module for scraping, parsing, and ingesting MLB GameDay data into a database',
  author = 'Chris Anderson',
  author_email = 'christian.c.anderson@gmail.com',
  url = 'https://github.com/raveneyes/pygameday',
  download_url = 'https://github.com/raveneyes/pygameday/tarball/0.1',
  keywords = ['baseball', 'gameday', 'database', 'scraping'],
  classifiers = [],
)
